import { html, css, LitElement, unsafeCSS, TemplateResult, PropertyValueMap } from 'lit';
import mainStyles from './style.js';

import icons from './assets/VectorGraphics.js';

import { customElement, property } from 'lit/decorators.js';
import { query } from 'lit/decorators/query.js';

import { LitElementWw } from '@webwriter/lit';
import { MoleculeCanvas } from './components/MoleculeCanvas.js';
import { IconButton } from './components/Button/IconButton.js';
import { TextButton } from './components/Button/TextButton.js';

import SlButton from '@shoelace-style/shoelace/dist/components/button/button.component.js';
import SlTooltip from '@shoelace-style/shoelace/dist/components/tooltip/tooltip.component.js';

import {
    faPlus,
    biType,
    biMarkerTip,
    biHiglighter,
    biTypeItalic,
    biTypeBold,
    biEraser,
    cSingle,
    cDouble,
    cTripple,
    biEyeDropper,
    biArrowLeft,
} from './icons.js';
import { PeriodicTable } from './components/PeriodicTable.component';

@customElement('webwriter-chemdraw')
export class ChemDraw extends LitElementWw {
    public static get styles() {
        return [mainStyles];
    }

    @property({ type: Number, attribute: false, reflect: false })
    accessor zoom: number = 6;

    @property({ type: Boolean }) accessor bold = false;
    @property({ type: Boolean }) accessor italic = false;
    @property({ type: Boolean }) accessor underline = false;
    @property({ type: Boolean }) accessor delete = false;
    @property({ type: Boolean }) accessor lewis = false;
    @property({ type: Boolean }) accessor color = false;
    @property({ type: Boolean }) accessor highlight = false;
    @property({ type: Array }) accessor elements = new Array();
    @property({ type: String }) accessor insertMode = 'H';
    @property({ type: Number }) accessor bondMode = 1;
    @property({ type: Boolean }) accessor elementselect = false;

    @property({ type: Object, attribute: true, reflect: true }) accessor molecule: {
        elements: {
            x: number;
            y: number;
            label: string;
            style: number;
            color: string;
            background: string;
            deco: any[];
        }[];
        bonds: { s: number; t: number; u: number; v: number; type: number }[];
    } = {
        elements: [],
        bonds: [],
    };

    @property({ type: Array })
    accessor canvasList: Array<any> = [];

    @query('#toolboxButtons') accessor toolboxButtons: HTMLElement;

    @query('#moleculeCanvas') accessor moleculeCanvas: MoleculeCanvas;

    constructor() {
        super();
        this.zoom = 6;
        this.insertMode = 'H';
        this.bondMode = 1;
        this.canvasList = [{}];
        this.elements = [
            {
                name: 'H',
                description: 'Wasserstoff',
            },
            {
                name: 'C',
                description: 'Kohlenstoff',
            },
            {
                name: 'N',
                description: 'Stickstoff',
            },
            {
                name: 'O',
                description: 'Sauerstoff',
            },
            {
                name: 'S',
                description: 'Schwefel',
            },
            {
                name: 'F',
                description: 'Flour',
            },
            {
                name: 'P',
                description: 'Phosphor',
            },
            {
                name: 'Cl',
                description: 'Chlor',
            },
            {
                name: 'Br',
                description: 'Brom',
            },
            {
                name: 'I',
                description: 'Iod',
            },
        ];

        this.addEventListener('wheel', (e) => {
            e.preventDefault();

            if (e.deltaY > 0) {
                if (this.zoom < 42) {
                    this.zoom++;
                }
            } else {
                if (this.zoom > 6) {
                    this.zoom--;
                }
            }
        });
    }

    static shadowRootOptions = { ...LitElement.shadowRootOptions, delegatesFocus: true };

    public static get scopedElements() {
        return {
            'ww-chem-icon-button': IconButton,
            'ww-chem-text-button': TextButton,
            'ww-chem-molecule-canvas': MoleculeCanvas,
            'webwriter-periodic-table': PeriodicTable,
            'sl-button': SlButton,
            'sl-tooltip': SlTooltip,
        };
    }

    render() {
        return html`
            ${this.toolboxTemplate()}
            <div class="flex-column">
                <div class="menu-bar horizontal flex-row" id="menu-tools" style="display:none">
                    <div class="sub-menu" id="sub-menu-view">
                        <!-- <ww-chem-icon-button
                            toggle
                            tooltip="Ansicht &auml;ndern: Skelettschreibweise"
                            iconName="skelett"
                            ?active=${!this.lewis}
                            @ww-chem-toggle=${(e) => {
                            this.lewis = !e.detail.value;
                        }}
                        ></ww-chem-icon-button>
                        <ww-chem-icon-button
                            toggle
                            active
                            tooltip="Ansicht &auml;ndern: Lewis-Schreibweise"
                            iconName="lewis"
                            ?active=${this.lewis}
                            @ww-chem-toggle=${(e) => {
                            this.lewis = e.detail.value;
                        }}
                        ></ww-chem-icon-button> -->
                        <!-- <ww-chem-icon-button
                            tooltip="Heranzoomen"
                            iconName="zoom_in"
                            @ww-chem-click=${() => {
                            if (this.zoom > 6) {
                                this.zoom--;
                            }
                        }}
                        ></ww-chem-icon-button>
                        <ww-chem-icon-button
                            tooltip="Rauszoomen"
                            iconName="zoom_out"
                            @ww-chem-click=${() => {
                            if (this.zoom < 42) {
                                this.zoom++;
                            }
                        }}
                        ></ww-chem-icon-button> -->
                    </div>
                    <div class="sub-menu" id="sub-menu-undo-redo">
                        <!-- <ww-chem-icon-button
                            toggle
                            tooltip="Textfarbe"
                            iconName="textcolor"
                            @ww-chem-toggle=${(e) => (this.color = e.detail.value)}
                        ></ww-chem-icon-button>
                        <ww-chem-icon-button
                            toggle
                            tooltip="Farbig hinterlegen"
                            iconName="highlight"
                            @ww-chem-toggle=${(e) => (this.highlight = e.detail.value)}
                        ></ww-chem-icon-button>
                        <ww-chem-icon-button
                            toggle
                            tooltip="Fett"
                            iconName="bold"
                            @ww-chem-toggle=${(e) => (this.bold = e.detail.value)}
                        ></ww-chem-icon-button>
                        <ww-chem-icon-button
                            toggle
                            tooltip="Kurisv"
                            iconName="italic"
                            @ww-chem-toggle=${(e) => (this.italic = e.detail.value)}
                        ></ww-chem-icon-button> -->
                    </div>
                    <div class="sub-menu" id="sub-menu-modes">
                        <!-- <ww-chem-icon-button
                            toggle
                            ?active=${!this.delete}
                            tooltip="Bearbiten"
                            iconName="cursor"
                            @ww-chem-toggle=${(e) => (this.delete = !e.detail.value)}
                        ></ww-chem-icon-btton>
                        <ww-chem-icon-button
                            toggle
                            ?active=${this.delete}
                            tooltip="L&oumlschen"
                            iconName="eraser"
                            @ww-chem-toggle=${(e) => (this.delete = e.detail.value)}
                        ></ww-chem-icon-butto> -->
                    </div>
                </div>
                <div class="flex-row">
                    <div class="content flex-row">
                        <ww-chem-molecule-canvas
                            .molecule=${this.molecule}
                            @ww-chem-molecule-change=${(e: any) => this.molecule = e.target.molecule}
                            @ww-chem-reactsTo=${(e) => {
                                this.canvasList = [
                                    ...this.canvasList,
                                    {
                                        class: e.detail.value,
                                    },
                                ];
                            }}
                            zoom=${this.zoom}
                            ?delete=${this.delete}
                            ?bold=${this.bold}
                            ?italic=${this.italic}
                            ?delete=${this.delete}
                            ?lewis=${this.lewis}
                            ?textColor=${this.color}
                            ?highlightColor=${this.highlight}
                            insertMode=${this.insertMode}
                            bondMode=${this.bondMode}
                            id="moleculeCanvas"
                        ></ww-chem-molecule-canvas>
                    </div>
                    <div class="menu-bar vertical flex-column" id="menu-objects" style="display:none">
                        <div class="sub-menu" id="sub-menu-elements">
                            ${this.elements.map((element) => {
                                return html`
                                    <ww-chem-text-button
                                        toggle
                                        ?active=${this.insertMode === element.name}
                                        tooltip=${element.description}
                                        displayText=${element.name}
                                        @ww-chem-toggle=${(e) => {
                                            if (e.detail.value) {
                                                this.insertMode = element.name;
                                            } else if (this.insertMode === element.name) {
                                                this.insertMode = null as any;
                                            }
                                        }}
                                    ></ww-chem-text-button>
                                `;
                            })}

                            <ww-chem-icon-button
                                tooltip="Neues Element"
                                iconName="add"
                                @ww-chem-click=${() => {
                                    const elementSymbol = prompt('Element Symbol');
                                    if (!elementSymbol) return alert('Element Symbol is required');
                                    const elementDescription = prompt('Element Description');
                                    if (!elementDescription) return alert('Element Description is required');
                                    this.elements = [
                                        ...this.elements,
                                        {
                                            name: elementSymbol,
                                            description: elementDescription,
                                        },
                                    ];
                                }}
                            ></ww-chem-icon-button>
                        </div>
                        <div class="sub-menu" id="sub-menu-verbindungen">
                            <ww-chem-icon-button
                                toggle
                                tooltip="Einfachbindung"
                                iconName="single"
                                ?active=${this.bondMode === 1}
                                @ww-chem-toggle=${(e) => {
                                    if (e.detail.value) {
                                        this.bondMode = 1;
                                    } else if (this.bondMode === 1) {
                                        this.bondMode = null as any;
                                    }
                                }}
                            ></ww-chem-icon-button>
                            <ww-chem-icon-button
                                toggle
                                tooltip="Doppelbindung"
                                iconName="double"
                                ?active=${this.bondMode === 2}
                                @ww-chem-toggle=${(e) => {
                                    if (e.detail.value) {
                                        this.bondMode = 2;
                                    } else if (this.bondMode === 2) {
                                        this.bondMode = null as any;
                                    }
                                }}
                            ></ww-chem-icon-button>
                            <ww-chem-icon-button
                                toggle
                                tooltip="Dreifachbindung"
                                iconName="triple"
                                ?active=${this.bondMode === 3}
                                @ww-chem-toggle=${(e) => {
                                    if (e.detail.value) {
                                        this.bondMode = 3;
                                    } else if (this.bondMode === 3) {
                                        this.bondMode = null as any;
                                    }
                                }}
                            ></ww-chem-icon-button>
                        </div>
                    </div>
                </div>
            </div>
            <webwriter-periodic-table
                class="periodic-table"
                style=${this.elementselect ? 'display: block;' : 'display: none;'}
                @element-click=${(e) => {
                    this.insertMode = e.detail.element.symbol;
                }}
            ></webwriter-periodic-table>
        `;
    }

    private toolboxTemplate(): TemplateResult {
        return html`
            <div class="toolbox">
                <sl-button
                    size="large"
                    circle
                    class="toolbox__open"
                    @click=${() => {
                        if (this.elementselect) {
                            this.elementselect = false;
                        } else {
                            this.openToolbox();
                        }
                    }}
                    id="toolboxButton"
                >
                    ${this.elementselect ? biArrowLeft : faPlus}
                </sl-button>

                <div class="toolbox__buttons closed" id="toolboxButtons">
                    <div class="toolbox__buttongroup">
                        <!-- <sl-tooltip content="Host" placement="left"> -->
                        <sl-button circle class="toolbox__btn"> ${biType} </sl-button>
                        <!-- </sl-tooltip> -->
                        <div class="toolbox__subbuttons">
                            <sl-tooltip content="Highlight Cell" placement="top">
                                <sl-button
                                    circle
                                    class="toolbox__btn"
                                    variant=${this.highlight ? 'primary' : 'default'}
                                    @click=${() => {
                                        this.highlight = !this.highlight;
                                    }}
                                >
                                    ${biMarkerTip}
                                </sl-button>
                            </sl-tooltip>
                            <sl-tooltip content="Highlight Text" placement="top">
                                <sl-button
                                    circle
                                    class="toolbox__btn"
                                    variant=${this.color ? 'primary' : 'default'}
                                    @click=${() => {
                                        this.color = !this.color;
                                    }}
                                >
                                    ${biHiglighter}
                                </sl-button>
                            </sl-tooltip>
                            <sl-tooltip content="Italic" placement="top">
                                <sl-button
                                    circle
                                    class="toolbox__btn"
                                    variant=${this.italic ? 'primary' : 'default'}
                                    @click=${() => {
                                        this.italic = !this.italic;
                                    }}
                                >
                                    ${biTypeItalic}
                                </sl-button>
                            </sl-tooltip>
                            <sl-tooltip content="Bold" placement="top">
                                <sl-button
                                    circle
                                    class="toolbox__btn"
                                    variant=${this.bold ? 'primary' : 'default'}
                                    @click=${() => {
                                        this.bold = !this.bold;
                                    }}
                                >
                                    ${biTypeBold}
                                </sl-button>
                            </sl-tooltip>
                        </div>
                    </div>
                    <div class="toolbox__buttongroup">
                        <!-- <sl-tooltip content="Host" placement="left"> -->
                        <sl-button
                            circle
                            class="toolbox__btn"
                            variant=${this.delete ? 'primary' : 'default'}
                            @click=${() => {
                                this.delete = !this.delete;
                            }}
                        >
                            ${biEraser}
                        </sl-button>
                        <!-- </sl-tooltip> -->
                    </div>
                    <div class="toolbox__buttongroup">
                        <!-- <sl-tooltip content="Host" placement="left"> -->
                        <sl-button circle class="toolbox__btn">
                            ${this.bondMode === 1 ? cSingle : this.bondMode === 2 ? cDouble : cTripple}
                        </sl-button>
                        <!-- </sl-tooltip> -->
                        <div class="toolbox__subbuttons">
                            <sl-tooltip content="Einfachbindung" placement="top">
                                <sl-button
                                    circle
                                    class="toolbox__btn"
                                    variant=${this.bondMode === 1 ? 'primary' : 'default'}
                                    @click=${() => {
                                        this.bondMode = 1;
                                    }}
                                >
                                    ${icons.single}
                                </sl-button>
                            </sl-tooltip>
                            <sl-tooltip content="Doppelbindung" placement="top">
                                <sl-button
                                    circle
                                    class="toolbox__btn"
                                    variant=${this.bondMode === 2 ? 'primary' : 'default'}
                                    @click=${() => {
                                        this.bondMode = 2;
                                    }}
                                >
                                    ${icons.double}
                                </sl-button>
                            </sl-tooltip>
                            <sl-tooltip content="Dreierbindung" placement="top">
                                <sl-button
                                    circle
                                    class="toolbox__btn"
                                    variant=${this.bondMode === 3 ? 'primary' : 'default'}
                                    @click=${() => {
                                        this.bondMode = 3;
                                    }}
                                >
                                    ${icons.triple}
                                </sl-button>
                            </sl-tooltip>
                        </div>
                    </div>
                    <div class="toolbox__buttongroup">
                        <!-- <sl-tooltip content="Elementauswahl" placement="right"> -->
                        <sl-button
                            circle
                            class="toolbox__btn"
                            variant=${this.elementselect ? 'primary' : 'default'}
                            @click=${() => {
                                this.elementselect = !this.elementselect;
                                this.toolboxButtons.classList.add('closed');
                            }}
                        >
                            ${biEyeDropper}
                        </sl-button>
                        <!-- </sl-tooltip> -->
                        <div class="toolbox__subbuttons">
                            <sl-tooltip content="Wasserstoff" placement="top">
                                <sl-button
                                    circle
                                    class="toolbox__btn"
                                    variant=${this.insertMode === 'H' ? 'primary' : 'default'}
                                    @click=${() => {
                                        this.insertMode = 'H';
                                    }}
                                >
                                    H
                                </sl-button>
                            </sl-tooltip>
                            <sl-tooltip content="Kohlenstoff" placement="top">
                                <sl-button
                                    circle
                                    class="toolbox__btn"
                                    variant=${this.insertMode === 'C' ? 'primary' : 'default'}
                                    @click=${() => {
                                        this.insertMode = 'C';
                                    }}
                                >
                                    C
                                </sl-button>
                            </sl-tooltip>
                            <sl-tooltip content="Stickstoff" placement="top">
                                <sl-button
                                    circle
                                    class="toolbox__btn"
                                    variant=${this.insertMode === 'N' ? 'primary' : 'default'}
                                    @click=${() => {
                                        this.insertMode = 'N';
                                    }}
                                >
                                    N
                                </sl-button>
                            </sl-tooltip>
                            <sl-tooltip content="Sauerstoff" placement="top">
                                <sl-button
                                    circle
                                    class="toolbox__btn"
                                    variant=${this.insertMode === 'O' ? 'primary' : 'default'}
                                    @click=${() => {
                                        this.insertMode = 'O';
                                    }}
                                >
                                    O
                                </sl-button>
                            </sl-tooltip>
                            <sl-tooltip content="Schwefel" placement="top">
                                <sl-button
                                    circle
                                    class="toolbox__btn"
                                    variant=${this.insertMode === 'S' ? 'primary' : 'default'}
                                    @click=${() => {
                                        this.insertMode = 'S';
                                    }}
                                >
                                    S
                                </sl-button>
                            </sl-tooltip>
                            <sl-tooltip content="Flour" placement="top">
                                <sl-button
                                    circle
                                    class="toolbox__btn"
                                    variant=${this.insertMode === 'F' ? 'primary' : 'default'}
                                    @click=${() => {
                                        this.insertMode = 'F';
                                    }}
                                >
                                    F
                                </sl-button>
                            </sl-tooltip>
                            <sl-tooltip content="Phosphor" placement="top">
                                <sl-button
                                    circle
                                    class="toolbox__btn"
                                    variant=${this.insertMode === 'P' ? 'primary' : 'default'}
                                    @click=${() => {
                                        this.insertMode = 'P';
                                    }}
                                >
                                    P
                                </sl-button>
                            </sl-tooltip>
                            <sl-tooltip content="Chlor" placement="top">
                                <sl-button
                                    circle
                                    class="toolbox__btn"
                                    variant=${this.insertMode === 'Cl' ? 'primary' : 'default'}
                                    @click=${() => {
                                        this.insertMode = 'Cl';
                                    }}
                                >
                                    Cl
                                </sl-button>
                            </sl-tooltip>
                            <sl-tooltip content="Brom" placement="top">
                                <sl-button
                                    circle
                                    class="toolbox__btn"
                                    variant=${this.insertMode === 'Br' ? 'primary' : 'default'}
                                    @click=${() => {
                                        this.insertMode = 'Br';
                                    }}
                                >
                                    Br
                                </sl-button>
                            </sl-tooltip>
                            <sl-tooltip content="Iod" placement="top">
                                <sl-button
                                    circle
                                    class="toolbox__btn"
                                    variant=${this.insertMode === 'I' ? 'primary' : 'default'}
                                    @click=${() => {
                                        this.insertMode = 'I';
                                    }}
                                >
                                    I
                                </sl-button>
                            </sl-tooltip>
                        </div>
                    </div>
                </div>
            </div>
        `;
    }

    private openToolbox(): void {
        this.toolboxButtons.classList.toggle('closed');
    }
}
